/* sndlibextra.c

   An interface to the low-level sndlib functions contained in sndlib/headers.c,
   designed for use by the PVC programs.
   (sndlibextra.c is a very stripped-down version of sndlibsupport.c in RTcmix.) 

   - John Gibson (jgg9c@virginia.edu)
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <math.h>
#include <errno.h>
#include <assert.h>
#include "sndlibextra.h"

/* #define NDEBUG */     /* define to disable asserts */


/* --------------------------------------------------- sndlib_read_header --- */
/* Reads the header of <fd> into the sndlib header buffer, so that we then
   can call sndlib functions (like c_snd_header_type, c_snd_data_format, etc.)
   to gather information from the header.

   When a header has been read into the header buffer, we call it the
   "current header". Any function in this file with "current_header" as
   part of its name depends on a previous call to sndlib_read_header.
*/
int
sndlib_read_header(int fd)
{
   assert(fd >= 0);

   /* Init the part of sndlib we need -- only inits once. */
   mus_create_header_buffer();

   if (lseek(fd, 0, SEEK_SET) == -1)
      return -1;

   mus_read_header_with_fd(fd);    /* NOTE: sndlib doesn't return status! */

   return 0;
}


/* -------------------------------------------------- sndlib_write_header --- */
/* (a wrapper for mus_write_header_with_fd in sndlib/headers.c)

   Writes a header to the newly created sound file opened as <fd>. If
   <data_location> is not NULL, sets the value it references to the offset
   of the start of sound data in the file. (This is computed by sndlib
   during the header write.)

   Assumes that <header_type> is one writeable by sndlib. (Find out in advance
   by using the WRITEABLE_HEADER_TYPE macro.) Caller is responsible for knowing
   if <header_type> supports <data_format> (including endianness concerns). If
   AIFF header type, make sure sndlib knows whether it's writing AIFF or AIFC
   (by calling sndlib_current_header_match_aiff_flavor (below) or the sndlib
   set_aifc_header function directly).

   CAUTION: This will write over the header of an existing sound file!
   This would almost certainly change the sound data start location, which
   might cause sound data loss or corruption (e.g., channel swapping or bad
   sample word alignment).

   IMPORTANT: Before closing this sound file, you must update the header's
   idea of how much sound data is in the file, by calling
   sndlib_set_header_data_size.

   On success, leaves file position pointer at end of header and returns 0;
   Otherwise, returns -1.
*/
int
sndlib_write_header(int  fd,
                    int  header_exists,      /* unused in this version */
                    int  header_type,
                    int  data_format,
                    int  srate,
                    int  chans,
                    char *comment,
                    int  *data_location)
{
   int  result, comment_len = 0;

   assert(fd >= 0 && header_type > -1 && data_format > 0);

   if (data_location)
      *data_location = 0;

   if (comment)
      comment_len = strlen(comment);

   if (lseek(fd, 0, SEEK_SET) == -1) {
      perror("sndlib_write_header: lseek");
      return -1;
   }

   mus_create_header_buffer();         /* in case this hasn't been done yet */

   /* Hopefully this succeeds, because sndlib doesn't really tell us! */
   result = mus_write_header_with_fd(fd, header_type, srate, chans, 0, 0,
                                     data_format, comment ? comment : "",
                                     comment_len);
   if (result == -1) {     /* as of sndlib-5.5, only if header_type is wrong */
      fprintf(stderr, "sndlib_write_header: Can't write header\n");
      return -1;
   }

   /* Data location computed during header write. */
   if (data_location)
      *data_location = mus_header_data_location();

   return 0;
}


/* ------------------------------------------ sndlib_set_header_data_size --- */
/* Update the header of file opened as <fd>, of type <header_type>, with
   the number of bytes of sound data written to the file, <sound_bytes).
*/
int
sndlib_set_header_data_size(int fd, int header_type, int sound_bytes)
{
   assert(fd >= 0 && header_type >= 0 && sound_bytes >= 0);

   /* Note: as of sndlib-5.5, this doesn't require us to make sure
      file's header is in sndlib header buffer. Nor do we have to
      seek to the beginning of the file first.
   */
   return mus_update_header_with_fd(fd, header_type, sound_bytes);
}


/* --------------------------------------------------------------------------- */

/* Copied from sndlib's sound.c to avoid compiling more sndlib code. */

#define HAVE_VPRINTF 1

#include <stdarg.h>

static void (*mus_error_handler)(int err_type, char *err_msg);
static char mus_error_buffer[256];

void mus_error(int error, char *format, ...)
{
#if HAVE_VPRINTF
   va_list ap;
   va_start(ap, format);
   vsprintf(mus_error_buffer, format,ap);
   va_end(ap);
   if (mus_error_handler)
      (*mus_error_handler)(error, mus_error_buffer);
   else fprintf(stderr, mus_error_buffer);
#else
   fprintf(stderr, "error: %d", error);
#endif
}


/* Copied from sndlib's audio.c to avoid compiling that file.
   (write_audio referenced from checked_write() in io.c)
*/
int write_audio(int line, char *buf, int bytes)
{
   return 0;
}

